﻿///////////////////////////////////////////////////////////////////////////////
//
// CsvRecordクラス
//  
// Copyright (c) 2016, Advanced Core Technologies, Inc. All Rights Reserved.
//
// 2016/01/13  Rev.1.0.0.0  MOTO    作成
// 2016/01/21  Rev.1.0.1.0  MOTO    ToString()を修正（エスケープ処理追加）
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;

namespace ACT.Utilities.CsvIO
{
    /// <summary>
    /// CSVファイルのレコードを表します。
    /// </summary>
    public class CsvRecord
    {
        //=========================================================================================
        //  Constructors
        //=========================================================================================
        #region Constructors

        /// <summary>
        /// 指定された区切り文字とトークンを使用して初期化します。
        /// </summary>
        /// <param name="delimiter">区切り文字</param>
        /// <param name="quote">引用符</param>
        /// <param name="tokens">トークンを格納するIEnumerable</param>
        public CsvRecord(Delimiter delimiter = Delimiter.Comma,
            QuotationMark quote = QuotationMark.Double, IEnumerable<string> tokens = null)
        {
            Delimiter = delimiter;
            QuotationMark = quote;

            if (tokens == null)
            {
                return;
            }
            foreach (var token in tokens)
            {
                Add(token);
            }
        }

        #endregion

        //=========================================================================================
        //  Private Constants
        //=========================================================================================
        #region Private Fields

        private static readonly char DoubleQuote = '"';

        #endregion

        //=========================================================================================
        //  Private Fields
        //=========================================================================================
        #region Private Fields

        /// <summary>
        /// トークンのバッキングフィールドです。
        /// </summary>
        private List<string> tokens_ = new List<string>();

        #endregion

        //=========================================================================================
        //  Properties
        //=========================================================================================
        #region Properties

        /// <summary>
        /// トークンの数を取得します。
        /// </summary>
        public int Count
        {
            get
            {
                return tokens_.Count;
            }
        }

        /// <summary>
        /// 区切り文字の種別を取得します。
        /// </summary>
        public Delimiter Delimiter { get; set; }

        /// <summary>
        /// 区切り文字を取得します。
        /// </summary>
        public char DelimiterCharacter
        {
            get
            {
                char delimiter = '\0';
                switch (Delimiter)
                {
                    case Delimiter.Comma:
                        delimiter = ',';
                        break;
                    case Delimiter.Tab:
                        delimiter = '\t';
                        break;
                }

                return delimiter;
            }
        }

        /// <summary>
        /// 引用符を取得または設定します。
        /// </summary>
        public QuotationMark QuotationMark { get; set; }

        /// <summary>
        /// 指定したインデックスのトークンを取得または設定します。
        /// </summary>
        /// <param name="index">インデックス番号</param>
        /// <returns>指定したインデックス位置にあるトークン。範囲外を指定した場合はnull。</returns>
        public string this[int index]
        {
            get
            {
                string token = null;
                if (index >= 0 && index < Count)
                {
                    token = tokens_[index];
                }

                return token;
            }
            set
            {
                if (index >= 0 && index < Count)
                {
                    tokens_[index] = value;
                }
            }
        }

        #endregion

        //=========================================================================================
        //  Public Methods
        //=========================================================================================
        #region Public Methods

        /// <summary>
        /// レコードの末尾にvalueを追加します。
        /// </summary>
        /// <param name="value">追加するトークン</param>
        public void Add(string value)
        {
            string token = value;
            if (token == null)
            {
                token = string.Empty;
            }
            tokens_.Add(token);
        }

        /// <summary>
        /// index番目のトークンを削除します。
        /// </summary>
        /// <param name="index">削除するインデックス番号</param>
        public void RemoveAt(int index)
        {
            if (index < 0 || index >= Count)
            {
                throw new ArgumentOutOfRangeException("index");
            }

            tokens_.RemoveAt(index);
        }

        /// <summary>
        /// 全てのトークンを削除します。
        /// </summary>
        public void Clear()
        {
            tokens_.Clear();
        }

        /// <summary>
        /// index番目にvalueのトークンを挿入します。
        /// </summary>
        /// <param name="index">挿入するインデックス位置</param>
        /// <param name="value">追加するトークン</param>
        public void Insert(int index, string value)
        {
            if (index < 0 || index >= Count)
            {
                throw new ArgumentOutOfRangeException("index");
            }

            tokens_.Insert(index, value);
        }

        /// <summary>
        /// インスタンスの値を文字列で返します。
        /// </summary>
        /// <returns>区切り文字で区切られた文字列。トークンがない場合は空文字。</returns>
        public override string ToString()
        {
            if (Count == 0)
            {
                return string.Empty;
            }

            StringBuilder builder = new StringBuilder();
            foreach (var token in tokens_)
            {
                // 二重引用符をエスケープし、引用符でトークンを囲い、末尾に区切り文字を追加
                string escaped = AddEscapeDoubleQuote(token);
                builder.Append(EnclosedWithinQuotes(escaped, QuotationMark));
                builder.Append(DelimiterCharacter);
            }
            // 最後の区切り文字を削除
            builder.Remove(builder.Length - 1, 1);

            return builder.ToString();
        }

        /// <summary>
        /// 全てのトークンを返します。
        /// </summary>
        /// <returns>トークンを格納した配列。トークンがない場合はnull。</returns>
        public string[] GetAllTokens()
        {
            if (Count == 0)
            {
                return null;
            }

            return tokens_.ToArray();
        }

        #endregion

        //=========================================================================================
        //  Private Methods
        //=========================================================================================
        #region Private Methods

        /// <summary>
        /// 引用符で囲んだ文字列を返します。
        /// </summary>
        /// <param name="value">対象の文字列</param>
        /// <param name="quote">引用符</param>
        /// <returns>引用符で囲んだ文字列。</returns>
        private static string EnclosedWithinQuotes(string value, QuotationMark quotationMark)
        {
            if (value == null)
            {
                throw new ArgumentNullException("value");
            }
            if (quotationMark == QuotationMark.None)
            {
                return value;
            }

            return DoubleQuote + value + DoubleQuote;
        }

        /// <summary>
        /// 二重引用符をエスケープした文字列を返します。
        /// </summary>
        /// <param name="value">対象の文字列</param>
        /// <returns>二重引用符をエスケープした文字列。</returns>
        private static string AddEscapeDoubleQuote(string value)
        {
            if (value == null)
            {
                throw new ArgumentNullException("value");
            }

            return value.Replace("\"", "\"\"");
        }

        #endregion
    }

    /// <summary>
    /// 区切り文字を指定します。
    /// </summary>
    public enum Delimiter
    {
        /// <summary>
        /// カンマで区切ります。
        /// </summary>
        Comma,
        /// <summary>
        /// タブ文字で区切ります。
        /// </summary>
        Tab,
    }

    /// <summary>
    /// 引用符の文字を指定します。
    /// </summary>
    public enum QuotationMark
    {
        /// <summary>
        /// 二重引用符を指定します。
        /// </summary>
        Double,
        /// <summary>
        /// 引用符なしを指定します。
        /// </summary>
        None,
    }
}
