﻿///////////////////////////////////////////////////////////////////////////////////////////////////
// 
//  ACT距離計算サービス サンプルプログラム
//  MainFormクラス
//
//  Copyrights (c) 2016-2023, Advanced Core Technologies, Inc. All rights reserved.
//
//  2023/09/21 Rev.1.0.0.1 KYON AdminURLに合わせてSSL使用フラグを変更出来るように変更
//  2016/01/26 Rev.1.0.0.0 MOTO 作成
//
///////////////////////////////////////////////////////////////////////////////////////////////////

using Act.Dcws;
using ACT.Utilities.CsvIO;
using System;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace ACTSOAPCSCalcOneWay
{
    public partial class MainForm : Form
    {
        //=========================================================================================
        //  Constructors
        //=========================================================================================
        #region Constructors

        /// <summary>
        /// 既定値を使用して初期化します。
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        #endregion

        //=========================================================================================
        //  Private Fields
        //=========================================================================================
        #region Private Fields

        /// <summary>
        /// ユーザー情報です。
        /// </summary>
        private UserSettings userSettings_;

        /// <summary>
        /// 距離計算サービスのユーザーです。
        /// </summary>
        private User user_;

        #endregion

        //=========================================================================================
        //  Private Constants
        //=========================================================================================
        #region Private Constants

        /// <summary>
        /// 経度の列番号です。
        /// </summary>
        private static readonly int LonColumnIndex = 2;

        /// <summary>
        /// 緯度の列番号です。
        /// </summary>
        private static readonly int LatColumnIndex = 3;

        #endregion

        //=========================================================================================
        //  Private Methods
        //=========================================================================================
        #region Private Methods

        /// <summary>
        /// 計算を実行し実行結果をファイルに出力します。
        /// </summary>
        /// <param name="userSettings">ユーザー設定</param>
        /// <returns>実行結果のメッセージ。</returns>
        private string DoCalc(UserSettings userSettings)
        {
            StringBuilder resultMessage = new StringBuilder();

            try
            {
                //---------------------------------------------------------------------------------
                //  管理サービスオブジェクトと環境設定オブジェクトの作成
                //---------------------------------------------------------------------------------
                // 管理サービスオブジェクトを作成
                DCAdmin.ACTDCADMIN adminService = new DCAdmin.ACTDCADMIN();
                // エンドポイントURLを設定
                adminService.Url = user_.AdminUrl;

                // 環境設定オブジェクトを作成
                DCAdmin.Environ adminEnviron = user_.GetAdminEnviron();

                //---------------------------------------------------------------------------------
                //  ユーザー情報オブジェクトの取得
                //---------------------------------------------------------------------------------
                // ユーザー情報オブジェクトを作成
                DCAdmin.UserInformation2 userInfo = new DCAdmin.UserInformation2();
                // ユーザー情報取得サービスの呼び出し
                if (!adminService.GetUserInformation2(adminEnviron, ref userInfo))
                {
                    // ユーザー情報の取得に失敗した場合
                    resultMessage.AppendLine("ユーザー情報の取得に失敗しました。");
                    return resultMessage.ToString();
                }
                user_.UserInfo = userInfo;

                //---------------------------------------------------------------------------------
                //  計算用道路データの取得と選択
                //---------------------------------------------------------------------------------
                // 計算用道路データ取得サービスの呼び出し           
                DCAdmin.NWInfo[] nwInfos = GetNwInfos(user_, adminService, adminEnviron);
                if (nwInfos == null)
                {
                    // 計算用道路データの取得に失敗した場合
                    resultMessage.AppendLine("計算用道路データの取得に失敗しました。");
                    return resultMessage.ToString();
                }

                // 距離計算に使用する計算用道路データを選択
                DCAdmin.NWInfo nwInfo = SelectNwInfo(nwInfos, CalcSettings.NWIDPattern);
                if (nwInfo == null)
                {
                    // 指定の計算用道路データが含まれていない場合
                    resultMessage.AppendLine(
                        "利用可能な計算用道路データに指定のデータが含まれていません。");
                    return resultMessage.ToString();
                }
                resultMessage.AppendLine("計算用道路データ: " + nwInfo.Note);

                //---------------------------------------------------------------------------------
                //  距離計算サービスオブジェクトと環境設定オブジェクトの作成
                //---------------------------------------------------------------------------------
                DCService.ACTDCWS dcService = new DCService.ACTDCWS();
                // 距離計算サーバー取得サービスの呼び出し
                if (!adminService.GetUsableDCServer(ref adminEnviron, nwInfo))
                {
                    // 距離計算サーバーの取得に失敗した場合
                    resultMessage.AppendLine("距離計算サーバーの情報取得に失敗しました。");
                    return resultMessage.ToString();
                }
                // エンドポイントURLを設定
                dcService.Url = Utilities.ChangeProtocol(adminEnviron.WSURL, CalcSettings.UseSsl);
                resultMessage.AppendLine("距離計算サーバー: " + dcService.Url +
                    "(" + adminEnviron.Host + ":" + adminEnviron.Port + ")");

                // 環境設定構造体オブジェクトを作成
                DCService.Environ dcEnvrion = Utilities.GetDCServiceEnviron(adminEnviron);

                //---------------------------------------------------------------------------------
                //  CSVファイルを読み込みノードポイントとロケーション構造体配列を作成
                //---------------------------------------------------------------------------------
                // ファイルからスタート地点の経度緯度情報を読み込む
                CsvRecord[] records = ReadCsvFile(userSettings.InputFilePath);
                if (records == null)
                {
                    // CSVファイルの読み込みに失敗した場合
                    resultMessage.AppendLine("入力ファイルの読み込みに失敗しました。");
                    return resultMessage.ToString();
                }
                else if (records.Length <= 1)
                {
                    // 2地点以上指定されていない場合
                    resultMessage.AppendLine("スタート地点を含む2点以上を指定してください。");
                    return resultMessage.ToString();
                }

                // 1行目のデータでスタート地点を設定するノードポイントを作成
                DCService.NodePnt startPnt = new DCService.NodePnt()
                {
                    Lon = records[0][LonColumnIndex],
                    Lat = records[0][LatColumnIndex],
                };

                // 2行目以降のデータでロケーション構造体配列を作成
                DCService.Loc[] locs =
                    CreateLocations(records.Copy<CsvRecord>(1, records.Length - 1));

                //---------------------------------------------------------------------------------
                //  片道一括計算用パラメータ構造体の設定
                //---------------------------------------------------------------------------------
                // 計算範囲を種別にあわせて換算（時間圏=0.1秒単位、距離圏=m単位）
                int areaRange = 0;
                int maxAreaRange = 0;
                switch (userSettings.AreaKind)
                {
                    // 時間圏の場合
                    case AreaKind.Time:
                        areaRange = userSettings.AreaRange * 60 * 10;
                        maxAreaRange = user_.UserInfo.CalcAreaMaxTime;
                        break;
                    // 距離圏の場合
                    case AreaKind.Dist:
                        areaRange = userSettings.AreaRange * 1000;
                        maxAreaRange = user_.UserInfo.CalcAreaMaxDist;
                        break;
                }
                // 計算範囲が最大値を超えていないか検証
                if (maxAreaRange != -1 && areaRange > maxAreaRange)
                {
                    resultMessage.AppendLine("計算範囲が最大値を超えています。");
                    return resultMessage.ToString();
                }

                // 片道一括計算用パラメータ構造体オブジェクトを作成
                DCService.COW3Param param = new DCService.COW3Param()
                {
                    Datum = (int)CalcSettings.Datum,
                    CoordinateFormat = CalcSettings.CoordinateFormat,
                    CalcKind = (int)CalcSettings.CalcKind,
                    StartPnt = startPnt,
                    AreaKind = (int)userSettings.AreaKind,
                    Transport = (int)CalcSettings.Transport,
                    AreaRange = areaRange,
                    AreaDirection = (int)userSettings.AreaDirection,
                    HighwayOutput = (int)CalcSettings.HighwayOutput,
                    TollOutput = (int)CalcSettings.TollOutput,
                };

                //---------------------------------------------------------------------------------
                //  片道一括計算の実行＆実行結果の取得
                //---------------------------------------------------------------------------------
                // 計算結果を格納する変数を作成
                DCService.NWInfo dcnwInfo = null;

                // 計算可能地点数を取得
                int maxLocs = user_.UserInfo.CalcAreaMaxLocs;

                // 必要計算回数を求める
                int numCalc = (int)Math.Ceiling((double)locs.Length / (double)maxLocs);

                // 分割して計算を実行
                for (int i = 0; i < numCalc; i++)
                {
                    // 対象地点を設定
                    int start = maxLocs * i;
                    int length = i + 1 == numCalc ? locs.Length - start : maxLocs;
                    DCService.Loc[] target = locs.Copy<DCService.Loc>(start, length);

                    // 片道一括計算の実行
                    int result = dcService.CalcOneWay3(dcEnvrion, param, ref target, ref dcnwInfo);

                    // 実行結果の取得
                    if (result == 0)
                    {
                        // 成功
                        resultMessage.AppendLine(
                            string.Format("{0} / {1} 回目：成功", i + 1, numCalc));
                    }
                    else
                    {
                        // 失敗、エラーメッセージを取得
                        resultMessage.AppendLine(string.Format("{0} / {1} 回目：失敗 {2}{3}",
                            i + 1, numCalc, Environment.NewLine,
                            dcService.GetDCWSMessage(dcEnvrion, result)));
                    }

                    // 実行結果を元の配列に書き戻す
                    Array.Copy(target, 0, locs, start, length);

                    // 最後のループ以外は待機
                    if (i + 1 != numCalc)
                    {
                        // ミリ秒単位の待機時間を取得
                        int waitTime = user_.UserInfo.CalcInterval * 1000;
                        // 待機
                        System.Threading.Thread.Sleep(waitTime);
                    }
                }

                // 最終距離計算日時を更新
                user_.UpdateLastDCTime();

                //---------------------------------------------------------------------------------
                //  ファイル出力
                //---------------------------------------------------------------------------------
                if (!WriteLocs(userSettings.OutputFilePath, locs, records))
                {
                    // 出力に失敗した場合
                    resultMessage.AppendLine("ファイルの書き込みに失敗しました。");
                }
            }
            catch (System.Net.WebException e)
            {
                resultMessage.AppendLine("通信エラーが発生しました。");
                resultMessage.AppendLine(e.Message);
            }
            catch
            {
                //
            }

            return resultMessage.ToString();
        }

        /// <summary>
        /// 計算を実行し実行結果をファイルに出力します。
        /// </summary>
        /// <param name="userSettings"></param>
        /// <returns>実行結果のメッセージ。</returns>
        private string DoWork(UserSettings userSettings)
        {
            string resultMessage = string.Empty;
            // 進捗フォームを表示して計算を実行
            ProgressDialog.Run(() => resultMessage = DoCalc(userSettings), "計算中");

            return resultMessage;
        }

        /// <summary>
        /// 計算用道路データ取得サービスを呼び出し、利用できる計算用道路データを取得します。
        /// </summary>
        /// <param name="user">計算用道路データ取得サービスに接続するユーザー</param>
        /// <returns></returns>
        private static DCAdmin.NWInfo[] GetNwInfos(User user,
            DCAdmin.ACTDCADMIN adminService, DCAdmin.Environ adminEnviron)
        {
            // 取得した計算用道路データを格納する配列を用意
            DCAdmin.NWInfo[] nwInfos = null;
            // 計算用道路データ取得サービスを呼び出す
            adminService.GetUsableNWInfos(adminEnviron, ref nwInfos);

            return nwInfos;
        }

        /// <summary>
        /// 指定した正規表現パターンとNWIDが一致するデータを返します。
        /// </summary>
        /// <param name="nwInfos">対象のNWInfo配列</param>
        /// <param name="pattern">NWIDの検索に使用する正規表現パターン</param>
        /// <returns>NWIDが一致したNWInfo。一致するNWInfoがない場合はnull。</returns>
        private static DCAdmin.NWInfo SelectNwInfo(DCAdmin.NWInfo[] nwInfos, string pattern)
        {
            // NWIDを正規表現で検索するクエリ
            var nwInfoQuery =
                from nwInfo in nwInfos
                where Regex.IsMatch(nwInfo.NWID.ToString(), CalcSettings.NWIDPattern)
                orderby nwInfo.NWID descending
                select nwInfo;

            // クエリの一番先頭を返す
            return nwInfoQuery.FirstOrDefault();
        }

        /// <summary>
        /// CSVファイルの全レコードを読み取り結果を返します。
        /// </summary>
        /// <param name="path">対象ファイルのパス</param>
        /// <returns>読み取り結果のCsvRecord配列。読み取りに失敗した場合はnull。</returns>
        private static CsvRecord[] ReadCsvFile(string path)
        {
            CsvRecord[] records = null;
            try
            {
                using (StreamReader strmReader =
                    new StreamReader(path, Encoding.GetEncoding("shift_jis")))
                using (CsvReader csvReader =
                    new CsvReader(strmReader, false, Delimiter.Comma, QuotationMark.Double))
                {
                    records = csvReader.ReadAllRecords();
                }
            }
            catch
            {
                //
            }

            return records;
        }

        /// <summary>
        /// CsvRecord配列からロケーション構造体配列を作成し返します。
        /// </summary>
        /// <param name="records">CsvRecord配列</param>
        /// <returns>ロケーション構造体配列。</returns>
        private static DCService.Loc[] CreateLocations(CsvRecord[] records)
        {
            // ロケーション構造体に経度緯度情報を設定
            DCService.Loc[] locations = new DCService.Loc[records.Length];
            for (int i = 0; i < records.Length; i++)
            {
                locations[i] = new DCService.Loc();
                locations[i].Lon = records[i][LonColumnIndex];
                locations[i].Lat = records[i][LatColumnIndex];
            }

            return locations;
        }

        /// <summary>
        /// ロケーション構造体の所要時間と距離をCSVファイルに書き込みます。
        /// </summary>
        /// <param name="path">出力ファイルのパス</param>
        /// <param name="locs">計算結果を格納したロケーション構造体配列</param>
        /// <param name="records">読み取り結果を格納したCsvRecord配列</param>
        /// <returns>書き込みに成功した場合はtrue、失敗した場合はfalse。</returns>
        private static bool WriteLocs(string path, DCService.Loc[] locs, CsvRecord[] records)
        {
            try
            {
                using (StreamWriter strmWriter =
                    new StreamWriter(path, false, Encoding.GetEncoding("shift_jis")))
                using (CsvWriter csvWriter = new CsvWriter(strmWriter))
                {
                    for (int i = 0; i < locs.Length; i++)
                    {
                        records[i + 1].Add(locs[i].Time.ToString());
                        records[i + 1].Add(((double)locs[i].Dist / 1000.0).ToString());

                        csvWriter.WriteRecord(records[i + 1]);
                    }
                }
            }
            catch
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// コントロールからユーザー設定を取得します。
        /// </summary>
        /// <returns>取得したUserSettingsオブジェクト。</returns>
        private UserSettings GetUserSettingsFromControls()
        {
            UserSettings settings = new UserSettings()
            {
                UserID = textBoxUserId.Text,
                Password = textBoxPassword.Text,
                AdminUrl = textBoxAdminServerUrl.Text,
                InputFilePath = textBoxInputFilePath.Text,
                OutputFilePath = textBoxOutputFilePath.Text,
                AreaKind = AreaKind.Time,
                AreaRange = (int)numericUpDownAreaRange.Value,
            };
            if (radioButtonInflow.Checked)
            {
                settings.AreaDirection = AreaDirection.InFlow;
            }
            if (radioButtonDist.Checked)
            {
                settings.AreaKind = AreaKind.Dist;
            }

            return settings;
        }

        /// <summary>
        /// 指定したユーザー設定をコントロールに適用します。
        /// </summary>
        /// <param name="settings">適用するUserSettingsオブジェクト</param>
        private void ApplyUserSettingsToControls(UserSettings settings)
        {
            textBoxUserId.Text = settings.UserID;
            textBoxAdminServerUrl.Text = settings.AdminUrl;
            textBoxInputFilePath.Text = settings.InputFilePath;
            textBoxOutputFilePath.Text = settings.OutputFilePath;
            if (settings.AreaDirection == AreaDirection.InFlow)
            {
                radioButtonInflow.Checked = true;
            }
            if (settings.AreaKind == AreaKind.Dist)
            {
                radioButtonDist.Checked = true;
            }
            numericUpDownAreaRange.Value = settings.AreaRange;
        }

        /// <summary>
        /// 入力ファイルを選択し、選択したファイルのパスを返します。
        /// </summary>
        /// <param name="oldPath">現在の入力ファイルパス</param>
        /// <returns>選択したファイルのパス。キャンセルの場合はnull。</returns>
        private static string SelectInputFile(string oldPath)
        {
            string newPath = null;
            using (OpenFileDialog dialog = new OpenFileDialog())
            {
                if (File.Exists(oldPath))
                {
                    dialog.InitialDirectory = Path.GetDirectoryName(oldPath);
                    dialog.FileName = Path.GetFileName(oldPath);
                }
                dialog.Title = "入力ファイルの選択";
                dialog.Filter = "CSVファイル (*.csv)|*.csv|全てのファイル (*.*)|*.*";

                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    newPath = dialog.FileName;
                }
            }

            return newPath;
        }

        /// <summary>
        /// 出力ファイルを選択し、選択したファイルのパスを返します。
        /// </summary>
        /// <param name="oldPath">現在の出力ファイルパス</param>
        /// <returns>選択したファイルのパス。キャンセルの場合はnull。</returns>
        private static string SelectOutputFile(string oldPath)
        {
            string newPath = null;
            using (SaveFileDialog dialog = new SaveFileDialog())
            {
                if (File.Exists(oldPath))
                {
                    dialog.InitialDirectory = Path.GetDirectoryName(oldPath);
                    dialog.FileName = Path.GetFileName(oldPath);
                }
                dialog.Title = "出力ファイルの選択";
                dialog.Filter = "CSVファイル (*.csv)|*.csv|全てのファイル (*.*)|*.*";

                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    newPath = dialog.FileName;
                }
            }

            return newPath;
        }

        /// <summary>
        /// ユーザー設定を検証します。
        /// </summary>
        /// <param name="userID">ユーザーID</param>
        /// <param name="password">パスワード</param>
        /// <param name="adminUrl">管理サーバーURL</param>
        /// <param name="inputFilePath">入力ファイルのパス</param>
        /// <param name="outputFilePath">出力ファイルのパス</param>
        /// <returns>設定が全て有効な場合はnull、それ以外の場合はエラーメッセージ。</returns>
        private static string ValidateSettings(string userID, string password, string adminUrl,
            string inputFilePath, string outputFilePath)
        {
            StringBuilder message = new StringBuilder();
            if (string.IsNullOrEmpty(userID))
            {
                message.AppendLine("ユーザーIDが指定されていません。");
            }
            if (string.IsNullOrEmpty(password))
            {
                message.AppendLine("パスワードが指定されていません。");
            }
            if (string.IsNullOrEmpty(adminUrl))
            {
                message.AppendLine("管理サーバーURLが指定されていません。");
            }
            string inputFilePathError = ValidateInputFilePath(inputFilePath);
            if (inputFilePathError != null)
            {
                message.AppendLine(inputFilePathError);
            }
            string outputFilePathError = ValidateOutputFilePath(outputFilePath);
            if (outputFilePathError != null)
            {
                message.AppendLine(outputFilePathError);
            }
            if (message.Length == 0)
            {
                if (ConfirmOverwrite(outputFilePath))
                {
                    return null;
                }
                message.AppendLine("指定された " + outputFilePath + " は既に存在します。");
            }

            return message.ToString();
        }

        /// <summary>
        /// 入力ファイルのパスを検証します。
        /// </summary>
        /// <param name="path">検査対象のパス</param>
        /// <returns>有効な場合はnull、それ以外の場合はエラーメッセージ。</returns>
        private static string ValidateInputFilePath(string path)
        {
            if (string.IsNullOrEmpty(path))
            {
                return "入力ファイルが指定されていません。";
            }
            try
            {
                if (!File.Exists(path))
                {
                    return "指定された " + path + " が存在しません。";
                }
            }
            catch
            {
                return "入力ファイルのパスが不正です。";
            }

            return null;
        }

        /// <summary>
        /// 出力ファイルのパスを検証します。
        /// </summary>
        /// <param name="path">検査対象のパス</param>
        /// <returns>有効な場合はnull、それ以外の場合はエラーメッセージ。</returns>
        private static string ValidateOutputFilePath(string path)
        {
            if (string.IsNullOrEmpty(path))
            {
                return "出力ファイルが指定されていません。";
            }
            try
            {
                string directoryName = Path.GetDirectoryName(path);
                if (string.IsNullOrEmpty(directoryName))
                {
                    return "指定されたフォルダが存在しません。";
                }
                if (!Directory.Exists(directoryName))
                {
                    return "指定された " + directoryName + " が存在しません。";
                }
            }
            catch
            {
                return "出力ファイルのパスが不正です。";
            }

            return null;
        }

        /// <summary>
        /// ファイルが既に存在する場合に上書き確認を行います。
        /// </summary>
        /// <param name="path">検査対象のパス</param>
        /// <returns>ファイルが既に存在していて上書きしない場合はtrue、それ以外の場合はfalse。</returns>
        private static bool ConfirmOverwrite(string path)
        {
            if (File.Exists(path))
            {
                DialogResult result = MessageBox.Show(path + " は既に存在しています。" +
                    Environment.NewLine + "上書きしてよろしいですか？", "上書き確認",
                    MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation,
                    MessageBoxDefaultButton.Button2);
                if (result == DialogResult.No)
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// 再計算可能までの残待機時間を更新します。
        /// </summary>
        /// <param name="waitingSeconds">秒単位の残り待機時間</param>
        private void UpdateWaitingTime(int waitingSeconds)
        {
            string labelText = string.Empty;
            if (waitingSeconds > 0)
            {
                // 待機時間が経過していない場合
                labelText = string.Format("再計算可能まで {0:00} 秒", waitingSeconds);
            }
            // コントロールの更新
            labelCountDown.Text = labelText;
            buttonCalc.Enabled = waitingSeconds <= 0;
            timerCalcWait.Enabled = waitingSeconds > 0;
        }

        #endregion

        //=========================================================================================
        //  Event Handlers
        //=========================================================================================
        #region Event Handlers

        /// <summary>
        /// メインフォームのロードイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // 保存したユーザー設定を復元し設定する
            userSettings_ = UserSettings.Load(UserSettings.SettingFilePath);
            if (userSettings_ == null)
            {
                // 失敗した場合は既定値を設定
                userSettings_ = new UserSettings();
                textBoxAdminServerUrl.Text = CalcSettings.DefaultAdminUrl;
                return;
            }
            ApplyUserSettingsToControls(userSettings_);
        }

        /// <summary>
        /// メインフォームのクローズ後イベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            // ユーザー設定を保存する
            userSettings_ = GetUserSettingsFromControls();
            try
            {
                userSettings_.Save(UserSettings.SettingFilePath);
            }
            catch
            {
                //
            }
        }

        /// <summary>
        /// 「既定値」ボタンのクリックイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonResetUrl_Click(object sender, EventArgs e)
        {
            textBoxAdminServerUrl.Text = CalcSettings.DefaultAdminUrl;
        }

        /// <summary>
        /// 「参照」ボタンのクリックイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSelectInputFile_Click(object sender, EventArgs e)
        {
            string path = SelectInputFile(textBoxInputFilePath.Text) ?? textBoxInputFilePath.Text;
            textBoxInputFilePath.Text = path;
        }

        /// <summary>
        /// 「参照」ボタンのクリックイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSelectOutputFile_Click(object sender, EventArgs e)
        {
            string path = SelectOutputFile(textBoxOutputFilePath.Text) ?? textBoxOutputFilePath.Text;
            textBoxOutputFilePath.Text = path;
        }

        /// <summary>
        /// 「時間（分）」ラジオボタンのチェック変更イベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void radioButtonTime_CheckedChanged(object sender, EventArgs e)
        {
            labelAreaKind.Text = radioButtonTime.Checked ? "分" : "km";
        }

        /// <summary>
        /// 「計算範囲」のテキスト変更イベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void numericUpDownAreaRange_TextChanged(object sender, EventArgs e)
        {
            if (numericUpDownAreaRange.Text == string.Empty)
            {
                numericUpDownAreaRange.Text = numericUpDownAreaRange.Value.ToString();
            }
        }

        /// <summary>
        /// 「計算」ボタンのクリックイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCalc_Click(object sender, EventArgs e)
        {
            // フォームから設定を取得し検証
            userSettings_ = GetUserSettingsFromControls();
            string errorMessage = ValidateSettings(userSettings_.UserID, userSettings_.Password,
                userSettings_.AdminUrl, userSettings_.InputFilePath, userSettings_.OutputFilePath);
            if (errorMessage != null)
            {
                textBoxResults.Text = errorMessage;
                return;
            }

            // テキストボックスの表示を変更
            textBoxResults.Text = "計算中...";

            // 「計算」ボタンを無効
            buttonCalc.Enabled = false;

            // AdminURLを確認してSSL使用フラグを変更
            CalcSettings.UseSsl = userSettings_.AdminUrl.StartsWith("https") == true;

            // ユーザーを作成
            user_ = new User(userSettings_.UserID, userSettings_.Password,
                userSettings_.AdminUrl, CalcSettings.UseSsl);

            // 計算の実行
            textBoxResults.Text = DoWork(userSettings_);

            // 再計算までのカウントダウン
            timerCalcWait.Start();
        }

        /// <summary>
        /// 「終了」ボタンのクリックイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// 「マニュアルを開く」リンクのクリックイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void linkLabelManual_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                Process.Start(UserSettings.UsersGuideFilePath);
            }
            catch
            {
                //
            }
        }

        /// <summary>
        /// 計算待機時間タイマーのイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timerCalcWait_Tick(object sender, EventArgs e)
        {
            if (user_.UserInfo == null)
            {
                UpdateWaitingTime(-1);
                return;
            }
            int elpsedSeconds = (int)((DateTime.Now - user_.LastDCTime).TotalSeconds);
            int waitingSeconds = user_.UserInfo.CalcInterval - elpsedSeconds;
            UpdateWaitingTime(waitingSeconds);
        }

        #endregion
    }
}
