﻿///////////////////////////////////////////////////////////////////////////////////////////////////
// 
//  ACT距離計算サービス サンプルプログラム
//  UserSettingsクラス
//
//  Copyrights (c) 2016 Advanced Core Technologies, Inc. All rights reserved.
//
//  2016/01/25 Rev.1.0.0.0 MOTO 作成
//
///////////////////////////////////////////////////////////////////////////////////////////////////

using Act.Dcws;
using System;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Xml;

namespace ACTSOAPCSCalcOneWay
{
    /// <summary>
    /// ユーザーの設定を保存、復元します。
    /// </summary>
    [DataContract]
    internal class UserSettings
    {
        //=========================================================================================
        //  Constructors
        //=========================================================================================
        #region Constructors

        /// <summary>
        /// 静的コンストラクターです。
        /// </summary>
        static UserSettings()
        {
            SettingFilePath = Path.Combine(AppPathDirectoryName, SettingFileName);
            UsersGuideFilePath = Path.Combine(AppPathDirectoryName, UsersGuideFileName);
        }

        #endregion

        //=========================================================================================
        //  Public Constants
        //=========================================================================================
        #region Public Constants

        /// <summary>
        /// 設定ファイルのパスです。
        /// </summary>
        public static readonly string SettingFilePath;

        /// <summary>
        /// ユーザーズガイドのパスです。
        /// </summary>
        public static readonly string UsersGuideFilePath;

        #endregion

        //=========================================================================================
        //  Private Constants
        //=========================================================================================
        #region Private Constants

        /// <summary>
        /// 実行ファイルのフォルダ名文字列です。
        /// </summary>
        private static readonly string AppPathDirectoryName =
            Path.GetDirectoryName(System.Windows.Forms.Application.ExecutablePath);

        /// <summary>
        /// 設定ファイルのファイル名です。
        /// </summary>
        private static readonly string SettingFileName = "CalcOneWay.config";

        /// <summary>
        /// ユーザーズガイドのファイル名です。
        /// </summary>
        private static readonly string UsersGuideFileName = "サンプルツール・ユーザーズガイド.pdf";

        #endregion

        //=========================================================================================
        //  Properties
        //=========================================================================================
        #region Properties

        /// <summary>
        /// 距離計算サービスに接続するユーザーIDを取得または設定します。
        /// </summary>
        [DataMember]
        public string UserID { get; set; }

        /// <summary>
        /// 距離計算サービスに接続するパスワードを取得または設定します。
        /// </summary>
        public string Password { get; set; }

        /// <summary>
        /// 距離計算サービスの管理サービスURLを取得または設定します。
        /// </summary>
        [DataMember]
        public string AdminUrl { get; set; }

        /// <summary>
        /// 入力ファイルのパスを取得または設定します。
        /// </summary>
        [DataMember]
        public string InputFilePath { get; set; }

        /// <summary>
        /// 出力ファイルのパスを取得または設定します。
        /// </summary>
        [DataMember]
        public string OutputFilePath { get; set; }

        /// <summary>
        /// 計算範囲の種別を取得または設定します。
        /// </summary>
        [DataMember]
        public AreaKind AreaKind { get; set; }

        /// <summary>
        /// 計算範囲を取得または設定します。
        /// </summary>
        [DataMember]
        public int AreaRange { get; set; }

        /// <summary>
        /// 範囲方向を取得または設定します。
        /// </summary>
        [DataMember]
        public AreaDirection AreaDirection { get; set; }

        #endregion

        //=========================================================================================
        //  Public Methods
        //=========================================================================================
        #region Public Methods

        /// <summary>
        /// このインスタンスをXMLシリアライズして、指定されたファイルに書き込みます。
        /// </summary>
        /// <param name="path">保存先のパス文字列</param>
        /// <returns>書き込みに成功した場合はtrue、それ以外の場合はfalse。</returns>
        public bool Save(string path)
        {
            if (string.IsNullOrWhiteSpace(path))
            {
                throw new ArgumentNullException("path");
            }
            string directoryName = Path.GetDirectoryName(path);
            if (!Directory.Exists(directoryName))
            {
                throw new DirectoryNotFoundException(path + "は存在しません。");
            }

            try
            {
                using (StreamWriter streamWriter =
                    new StreamWriter(path, false, new UTF8Encoding(false)))
                using (XmlWriter xmlWriter = XmlWriter.Create(streamWriter))
                {
                    DataContractSerializer serializer =
                        new DataContractSerializer(typeof(UserSettings));
                    serializer.WriteObject(xmlWriter, this);
                }
            }
            catch
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// 指定されたパスからUserSettingsオブジェクトをデシリアライズして返します。
        /// </summary>
        /// <param name="path">復元のパス文字列</param>
        /// <returns>デシリアライズしたUserSettingsオブジェクト、失敗した場合はnull。</returns>
        public static UserSettings Load(string path)
        {
            if (string.IsNullOrWhiteSpace(path))
            {
                throw new ArgumentNullException("path");
            }

            UserSettings settings = null;
            try
            {
                using (StreamReader streamReader = new StreamReader(path, new UTF8Encoding(false)))
                using (XmlReader xmlReader = XmlReader.Create(streamReader))
                {
                    DataContractSerializer seriazlier =
                        new DataContractSerializer(typeof(UserSettings));
                    settings = (UserSettings)seriazlier.ReadObject(xmlReader);
                }
            }
            catch
            {
                //
            }

            return settings;
        }

        #endregion
    }
}
